####################################################
####################################################
## This real data example is worked out in        ##
## slides 64-67. It is suitably adapted from      ##
## https://www4.stat.ncsu.edu/~reich/ABA/code/GLM ## 
##                                                ##
####################################################
####################################################

# remove all values from memory
rm(list=ls())

# load rjags library
library(rjags)


#################
# Load the data #
################# 
# Read data
load("gambia.RData")

# Save 'pos' to the response Y and save covariate in a matrix X
Y <- pos
n <- length(Y)
X <- as.matrix(X)

# Look at the response Y and the covariates X
table(Y)
head(X)

#####################
# Specify the model #
#####################
logistic_model <- "model{

   # Likelihood

   for(i in 1:n){
    Y[i] ~ dbern(q[i])
    logit(q[i]) <- beta[1] + beta[2]*X[i,1] + beta[3]*X[i,2] + 
                   beta[4]*X[i,3] + beta[5]*X[i,4] + beta[6]*X[i,5]
   }

   #Priors. Remember we specify SMALL PRECISION 0.01

   for(j in 1:6){
    beta[j] ~ dnorm(0,0.1)
   }

}"


#############################
# Compile the model in JAGS #
#############################

# Run 3 chains
dat   <- list(Y=Y,n=n,X=X)
model <- jags.model(textConnection(logistic_model),data = dat,n.chains=3, quiet=TRUE)


################
# Draw samples #
################

# First use function 'update' to draw 10,000 warm-up (burnin) samples
update(model, 10000, progress.bar="none")

# Next use the function 'coda.samples' to produce the next 10,000
# samples which will ultimately used to approximate the posterior.
samp <- coda.samples(model, 
                     variable.names=c("beta"), 
                     n.iter=10000, progress.bar="none")


##############################
# Trace plot and plot of the #
# approximate posteriors     #
##############################
graphics.off()
par("mar")
par(mar=c(3,3,3,3))
plot(samp)


########################################
# Summary statistics for the posterior #
########################################
summary(samp)

# Obtain the odds(beta_j), j=1,...,5 as these make the regression coefficients
# easier to interpret
beta1 <- samp[[1]]  # samples from chain 1
beta2 <- samp[[2]]  # samples from chain 2
beta3 <- samp[[3]]  # samples from chain 3

beta  <- rbind(beta1,beta2,beta3)

# Obtain the odds
odds  <- exp(beta)
colnames(odds)<-c("Int",colnames(X))
odds  <- odds[,-1]

# Posterior means
posterior.means <- apply(odds,2,mean)
# Posterior standard deviations
posterior.sds <- apply(odds,2,sd)
# Posterior credible intervals
posterior.creds <- apply(odds, 2, quantile, probs = c(0.025,0.975))

posterior.means
posterior.sds
posterior.creds

# Plots of the posterior odds ratios pi(odds(beta_j) | y), j=1,...,5
par(mfrow=c(2,3)) 
hist(odds[,1],breaks=50,main=colnames(X)[1],xlab="Odds",ylab="",yaxt="n")
hist(odds[,2],breaks=50,main=colnames(X)[2],xlab="Odds",ylab="",yaxt="n")
hist(odds[,3],breaks=50,main=colnames(X)[3],xlab="Odds",ylab="",yaxt="n")
hist(odds[,4],breaks=50,main=colnames(X)[4],xlab="Odds",ylab="",yaxt="n")
hist(odds[,5],breaks=50,main=colnames(X)[5],xlab="Odds",ylab="",yaxt="n")

# Export pdf of the plots of the posterior odds ratios
# pdf(file="odds_ratios_posteriors.pdf", height=6, width=8)
# par(mfrow=c(2,3)) 
# hist(odds[,1],breaks=50,main=colnames(X)[1],xlab="Odds",ylab="",yaxt="n")
# hist(odds[,2],breaks=50,main=colnames(X)[2],xlab="Odds",ylab="",yaxt="n")
# hist(odds[,3],breaks=50,main=colnames(X)[3],xlab="Odds",ylab="",yaxt="n")
# hist(odds[,4],breaks=50,main=colnames(X)[4],xlab="Odds",ylab="",yaxt="n")
# hist(odds[,5],breaks=50,main=colnames(X)[5],xlab="Odds",ylab="",yaxt="n")
# dev.off()