##########################################################
##########################################################
## This real data example is worked out in              ##
## slides 50-52. It is suitably adapted from            ##
## https://www4.stat.ncsu.edu/~reich/st590/code/regJAGS ## 
##                                                      ##
##########################################################
##########################################################

# remove all values from memory
rm(list=ls())

# load rjags library
library(rjags)

#################
# Prep the data #
################# 
# Read data
dat <- read.csv("Obama2012.csv")
head(dat)

# Y = percent of voters who voted for Obama in county
# white = percent white
# unemp = percent unemployed

Y <- 100*dat[,2]
# Standardize the response 'Y'
Y <- (Y-mean(Y))/sd(Y)
white <- dat[,7]
# Standardize the predictor 'white'
white <- (white-mean(white))/sd(white)
unemp <- dat[,18]
# Standardize the predictor 'unemp'
unemp <- (unemp-mean(unemp))/sd(unemp)
# Sample size n
n <- 100


#####################
# Specify the model #
#####################
model_string <- "model{

  # Likelihood
  for(i in 1:n){
    Y[i]   ~ dnorm(mu[i],inv.var)
    mu[i] <- beta[1] + beta[2]*white[i] + beta[3]*unemp[i]
  }

  # Prior for beta. Specify the PRECISION (should be SMALL)
  for(j in 1:3){
    beta[j] ~ dnorm(0,0.0001)
  }

  # Prior for the precision
  inv.var   ~ dgamma(0.01, 0.01)
  sigmasq   <- 1/inv.var

}"


#############################
# Compile the model in JAGS #
#############################

# Compile the model
model <- jags.model(textConnection(model_string), 
                    data = list(Y=Y,n=n,white=white,unemp=unemp),
                    n.chains=3,quiet=TRUE)


################
# Draw samples #
################

# First use function 'update' to draw 5000 warm-up (burnin) samples
update(model, 5000, progress.bar="none") # Burnin for 5000 samples

# Next use the function 'coda.samples' to produce the next 5000
# samples which will ultimately used to approximate the posterior.
samp <- coda.samples(model, 
                     variable.names=c("beta","sigmasq"), 
                     n.iter=5000, progress.bar="none")


##############################
# Trace plot and plot of the #
# approximate posteriors     #
##############################
graphics.off()
par("mar")
par(mar=c(3,3,3,3))
plot(samp)


########################################
# Summary statistics for the posterior #
########################################
summary(samp)