########################################################
########################################################
## This real data example is worked out in            ##
## slides 59-63. It is suitably adapted from          ##
## https://www4.stat.ncsu.edu/~reich/ABA/code/Poisson ## 
##                                                    ##
########################################################
########################################################

# remove all values from memory
rm(list=ls())

# load rjags library
library(rjags)

#################
# Load the data #
################# 
# Read data
load("guns.RData")

# Save the covariates in design matrix X
totlaws <- rowSums(X)
X <- cbind(Z,totlaws) 

n <- length(Y)
p <- ncol(X)

# Look at the response Y and the covariates X
head(Y) # Total number of gun-related homicides
head(N) # Total state population
head(X) # Covariates


#####################
# Specify the model #
#####################
model_string1 <- "model{

  # Likelihood
  for(i in 1:n){
    Y[i] ~ dpois(lambda[i])
    log(lambda[i]) <- log(N[i]) + alpha + inprod(beta[],X[i,])
  }

  #Priors
   for(j in 1:p){
      beta[j] ~ dnorm(0,0.1)
   }
   alpha ~ dnorm(0,0.1)
 }"


#############################
# Compile the model in JAGS #
#############################

# Run 3 chains
poisson.model <- jags.model(textConnection(model_string1), 
                     data = list(Y=Y,N=N,n=n,X=X,p=p),n.chains=3,
                     quiet=TRUE)


################
# Draw samples #
################

# First use function 'update' to draw 10000 warm-up (burnin) samples
update(poisson.model, 10000, progress.bar="none")

# Next use the function 'coda.samples' to produce the next 5000
# samples which will ultimately used to approximate the posterior.
samps <- coda.samples(poisson.model, 
                      variable.names=c("beta"), 
                      n.iter=10000, progress.bar="none")


##############################
# Trace plot and plot of the #
# approximate posteriors     #
##############################
graphics.off()
par("mar")
par(mar=c(3,3,3,3))
plot(samps)


########################################
# Summary statistics for the posterior #
########################################
summary(samps)