##########################################
##########################################
## This synthetic example is worked out ##
## in slides 33-36                      ##
##########################################
##########################################

# remove all values from memory
rm(list=ls())
# Load mvtnorm library so we can sample from multivariate normal density
library(mvtnorm)

# Set seed
set.seed(123456)


#################################################
#################################################
## Generate synthetic data of 500 observations ##
#################################################
#################################################

# True parameters
beta.true <- c(-3.3, 1.7, -1.9)

n <- 500

# Make design matrix with 500 rows, 3 columns
X <- matrix(0,n,3)
X[,1] <- rep(1,n) # First column is all 1's for intercept
X[,2] <- rnorm(n, mean=0, sd=1) # Fill in entries for first covariate x1
X[,3] <- rnorm(n, mean=0, sd=1) # Fill in entries for second covariate x2

# Generate responses y
X.betatrue <- X%*%beta.true
y <- rpois(n, exp(X.betatrue))

# Visualize the data
y
hist(y, breaks=80)


#######################################
#######################################
## Run Metropolis-Hastings algorithm ##
#######################################
#######################################

# Initialize values for algorithm
beta.init <- rep(0,3)

# number of MCMC samples and burnin
niter <- 5000
burnin <- 2500

# known hyperparameters tau0^2. These do not change
tau0sq <- 100

# covariance matrix V for the proposal density
inv.XtX <- solve(t(X)%*%X)
logy <- log(y+1/2)
sigma.hat <- (sd(logy))^2
V <- sigma.hat*inv.XtX

# Set seed to get same values each time running this script
set.seed(123456)

# Vectors to hold the MCMC output
beta0 <- rep(NA,niter+1)
beta1 <- rep(NA,niter+1)
beta2 <- rep(NA,niter+1)

# Set the first value of the chain to the initial value
beta <- as.matrix(beta.init)
beta0[1] <- beta.init[1]
beta1[1] <- beta.init[2]
beta2[1] <- beta.init[3]

# Run the Metropolis-Hastings algorithm which iteratively updates beta
for (t in 2:(niter+1)){
  
 # Draw proposal from N(beta,V)
  beta.star <- t(rmvnorm(1, mean=beta, sigma=V))

  # Compute X%*%beta
  X.beta <- X%*%beta
  # Compute X%*%beta.star
  X.beta.star <- X%*%beta.star
  
 # Calculate acceptance probability. Evaluate ratio on log-scale first to avoid numerical issues
 # Log of the numerator
 log.num.ratio <- sum(log(dpois(y,exp(X.beta.star))))+sum(log(dnorm(beta.star,mean=0,sd=sqrt(tau0sq))))+log(dmvnorm(as.vector(beta),mean=as.vector(beta.star),sigma=V))
 # Log of the denominator
 log.den.ratio <- sum(log(dpois(y,exp(X.beta))))+sum(log(dnorm(beta,mean=0,sd=sqrt(tau0sq))))+log(dmvnorm(as.vector(beta.star),mean=as.vector(beta),sigma=V))
 log.ratio <- log.num.ratio-log.den.ratio                                                                                                 
  
 # Finally compute acceptance probability alpha
 alpha <- min(1, exp(log.ratio))
 
 # Draw U from uniform(0,1)
 U <- runif(1,0,1)
 # Set beta = beta.star if U < alpha. Otherwise beta stays at the previous iteration's value
 if(U <= alpha)
   beta <- beta.star
 
 # Store values of the regression coefficients
 beta0[t] <- beta[1]
 beta1[t] <- beta[2]
 beta2[t] <- beta[3]
}


###############################
###############################
## Summarize the output from ##  
## the MH algorithm          ##
###############################
###############################

# save beta0, beta1, beta2 but do not save the initial values
# First row is posterior samples for beta0
# Second row is posterior samples for beta1
# Third row is posterior samples for beta2
parameter.chains <- rbind(beta0[-(1)], beta1[-(1)], beta2[-(1)])

# Discard the first 2500 iterations as burn-in
parameter.chains <- parameter.chains[,(burnin+1):niter]

# Plot the posterior distribution for pi(beta0|y)
hist(parameter.chains[1,],breaks=35,xlab=expression(paste(beta[0])),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", beta[0],"| y)")))

# Plot the posterior distribution for pi(beta1|y)
hist(parameter.chains[2,],breaks=30,xlab=expression(paste(beta[1])),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", beta[1],"| y)")))

# Plot the posterior distribution for pi(sigmasq|y)
hist(parameter.chains[3,],breaks=30,xlab=expression(paste(beta[2])),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", beta[2],"| y)")))



## Make pdf's of the plots
# pdf(file="PoissonMH_regression_coefficients_plots.pdf", width=8, height=8)
# par(mfrow=c(2,1))
# hist(parameter.chains[2,],breaks=30,xlab=expression(paste(beta[1])),ylab="",yaxt="n",
#      main=expression(paste("Posterior for ", pi,"(", beta[1],"| y)")))
# hist(parameter.chains[3,],breaks=30,xlab=expression(paste(beta[2])),ylab="",yaxt="n",
#       main=expression(paste("Posterior for ", pi,"(", beta[2],"|y)")))
# dev.off()


########################
########################
## Summary statistics ##
########################
########################

# The 2500 saved iterations from the Gibbs sampler
beta0.saved <- parameter.chains[1,]
beta1.saved <- parameter.chains[2,]
beta2.saved <- parameter.chains[3,]

# Report summary statistics for beta0
# Posterior mean
beta0.mean <- mean(beta0.saved)
# Posterior standard deviation
beta0.sd <- sd(beta0.saved)
# 95% posterior credible interval
beta0.posterior.cred <- quantile(beta0.saved,c(0.025,0.975))

# beta0.mean
# beta0.sd
# beta0.posterior.cred

# Report summary statistics for beta1
# Posterior mean
beta1.mean <- mean(beta1.saved)
# Posterior standard deviation
beta1.sd <- sd(beta1.saved)
# 95% posterior credible interval
beta1.posterior.cred <- quantile(beta1.saved,c(0.025,0.975))

beta1.mean
beta1.sd
beta1.posterior.cred

# Report summary statistics for sigmasq
# Posterior mean
beta2.mean <- mean(beta2.saved)
# Posterior standard deviation
beta2.sd <- sd(beta2.saved)
# 95% posterior credible interval
beta2.posterior.cred <- quantile(beta2.saved,c(0.025,0.975))

beta2.mean
beta2.sd
beta2.posterior.cred
