######################################################################
######################################################################
## This real data example is worked out in slides 53-58.            ##
## It is suitably adapted from                                      ##
## https://www4.stat.ncsu.edu/~reich/BSMdata/random_effect_jaw.html ## 
##                                                                  ##
######################################################################
######################################################################

# Change work directory 
# setwd("C:/Users/rayba/Dropbox/Postdoc at UPenn/GCB533. Statistics for Genomics Guest Lecture/Computing")

# remove all values from memory
rm(list=ls())

# load rjags library
library(rjags)

#################
# Prep the data #
################# 
m   <- 4 # number of observations per patient
n   <- 20 # number of patients
age <- c(8.0, 8.5, 9.0, 9.5)
Y   <- c(47.8, 48.8, 49.0, 49.7,
         46.4, 47.3, 47.7, 48.4,
         46.3, 46.8, 47.8, 48.5,
         45.1, 45.3, 46.1, 47.2,
         47.6, 48.5, 48.9, 49.3,
         52.5, 53.2, 53.3, 53.7,
         51.2, 53.0, 54.3, 54.5,
         49.8, 50.0, 50.3, 52.7,
         48.1, 50.8, 52.3, 54.4,
         45.0, 47.0, 47.3, 48.3,
         51.2, 51.4, 51.6, 51.9,
         48.5, 49.2, 53.0, 55.5,
         52.1, 52.8, 53.7, 55.0,
         48.2, 48.9, 49.3, 49.8,
         49.6, 50.4, 51.2, 51.8,  
         50.7, 51.7, 52.7, 53.3,
         47.2, 47.7, 48.4, 49.5,
         53.3, 54.6, 55.1, 55.3,
         46.2, 47.5, 48.1, 48.4,
         46.3, 47.6, 51.3, 51.8) 

Y <- matrix(Y,20,4,byrow=TRUE)


#####################
# Specify the model #
#####################

data     <- list(Y=Y,n=n,m=m) # Needs to be formatted this way for repeated measures data
burn     <- 5000   # burn-in
n.iter   <- 5000   # number of iterations after burnin
n.chains <- 2      # This time we will run 2 chains in parallel


# Specify the model
model_string <- textConnection("model{

   # Likelihood
    for(i in 1:n){for(j in 1:m){
      Y[i,j] ~ dnorm(alpha[i],inv.sigmasq)
    }}

   # Random effects
    for(i in 1:n){alpha[i] ~ dnorm(mu,inv.lambdasq)}

   # Priors. Recall for normal density, we specify SMALL PRECISION rather than large variance
    mu ~ dnorm(0,0.0001) 
    inv.sigmasq ~ dgamma(0.01,0.01)
    sigmasq   <- 1/inv.sigmasq
    inv.lambdasq ~ dgamma(0.01,0.01)
    lambdasq   <- 1/inv.lambdasq
 }")


#############################
# Compile the model in JAGS #
#############################
params   <- c("mu","alpha","sigmasq","lambdasq")
model    <- jags.model(model_string, data = data, 
                       n.chains=n.chains, quiet=TRUE)


################
# Draw samples #
################
# First use function 'update' to draw 5000 warm-up (burnin) samples
update(model, burn, progress.bar="none")

# Next use the function 'coda.samples' to produce the next 5000
# samples which will ultimately used to approximate the posterior.

samples1 <- coda.samples(model, variable.names=params, 
                         n.iter=n.iter, progress.bar="none")


##############################
# Trace plot and plot of the #
# approximate posteriors     #
##############################
graphics.off()
par("mar")
par(mar=c(3,3,3,3))
plot(samples1)

# Bind the samples from the two separate chains
samples1 <- rbind(samples1[[1]],samples1[[2]])

# Save the samples for the random effects alpha_i, population mean mu, 
# within-subject and between-subject variations, and the ratio of between-subject variation/total variation

alpha <- samples1[,1:n]             # Random effects
mu <- samples1[,n+2]                # Population mean 
all.vars  <- samples1[,c(n+1,n+3)]     # First column is sigmasq and second is lambdasq
r <- all.vars[,2]/rowSums(all.vars) # Proportion of variance explained by the random effect

# Plot of the proportion of variance explained by the random effect
graphics.off()
par("mar")
par(mar=c(4,4,4,4))
hist(r,breaks=50,prob=TRUE,main="",xlab="Proportion of variance explained by the random effect",ylab="",yaxt="n")


# pdf(file="proportion_random_effects.pdf", height=8, width=8)
# hist(r,breaks=50,prob=TRUE,main="",xlab="Proportion of variance explained by the random effect",ylab="",yaxt="n")
# dev.off()

# Plot of the random effects
boxplot(alpha~col(alpha),ylim=range(Y),xlab="Patient",ylab="Bone density",outline=FALSE)
points(row(Y),Y,pch=19)

# pdf(file="plot_random_effects.pdf", height=8, width=8)
# boxplot(alpha~col(alpha),ylim=range(Y),xlab="Patient",ylab="Bone density",outline=FALSE)
# points(row(Y),Y,pch=19)
# dev.off()

########################################
# Summary statistics for the posterior #
########################################

# Posterior means for mu, lambdasq, and r
mu.mean <- mean(mu)
lambdasq.mean <- mean(all.vars[,2])
r.mean <- mean(r)

# Posterior standard deviations for mu, lambdasq, and r
mu.sd <- sd(mu)
lambdasq.sd <- sd(all.vars[,2])
r.sd <- sd(r)

# 95% posterior credible intervals
mu.posterior.cred <- quantile(mu,c(0.025,0.975))
lambdasq.posterior.cred <- quantile(all.vars[,2],c(0.025,0.975))
r.posterior.cred <- quantile(r,c(0.025,0.975))

# Display results
mu.mean
lambdasq.mean
r.mean

mu.sd
lambdasq.sd
r.sd

mu.posterior.cred
lambdasq.posterior.cred
r.posterior.cred

