##########################################
##########################################
## This synthetic example is worked out ## 
## in slides 28-32 and 43-44            ##
##########################################
##########################################

# remove all values from memory
rm(list=ls())

# load rjags library
library(rjags)

# Set seed so we get the same result every time
set.seed(123456)


#################################################
#################################################
## Generate synthetic data of 500 observations ##
#################################################
#################################################

# True parameters
beta0.true <- 0.6
beta1.true <- -2.3
sigmasq.true <- 1.5

n <- 500
x <- rnorm(n, mean=0, sd=2)
# Note in R, you specify the standard dev, not the variance for the normal distribution
y <- beta0.true+beta1.true*x+rnorm(n,mean=0,sd=sqrt(sigmasq.true))

# Plot the data
plot(x,y, main=expression(paste(beta[0],"=",0.6,", ",beta[1],"=",-2.3,", ",sigma^2,"=",1.5)))


##################################
##################################
## Run Gibbs sampling algorithm ##
##################################
##################################

# Initialize values for algorithm
beta0.init <- 0
beta1.init <- 0
sigmasq.init <- 1

# number of MCMC samples and burnin
niter <- 5000
burnin <- 2500

# known hyperparameters tau0^2, tau1^2, a0, b0 in the priors on beta0, beta1, sigmaq. These do not change
tau0sq <- 100
tau1sq <- 100
a0 <- 0.01
b0 <- 0.01

# Set seed to get same values each time running this script
set.seed(123456)

# Vectors to hold the MCMC output
beta0 <- rep(NA,niter+1)
beta1 <- rep(NA,niter+1)
sigmasq <- rep(NA,niter+1)

# Set the first value of the chain to the initial value
beta0[1] <- beta0.init
beta1[1] <- beta1.init
sigmasq[1] <- sigmasq.init

# Run the Gibbs sampler which iteratively draws from the full conditional distributions
for (t in 2:(niter+1)){
  
  # mean for beta0 full conditional
  beta0.mean <- ( tau0sq*sum(y-beta1[t-1]*x) )/( sigmasq[t-1]+n*tau0sq )
  # variance for beta0 full conditional
  beta0.var <- ( sigmasq[t-1]*tau0sq )/( sigmasq[t-1]+n*tau0sq )
  
  # Sample from full conditional for beta0 and store it in the vector
  beta0[t] <- rnorm(1, mean=beta0.mean, sd=sqrt(beta0.var))
  
  # mean for beta1 full conditional
  beta1.mean <- ( tau1sq*sum(x*(y-beta0[t])) )/( sigmasq[t-1]+tau1sq*sum(x^2))
  # Variance for beta1 full conditional
  beta1.var <- ( sigmasq[t-1]*tau1sq )/( sigmasq[t-1]+tau1sq*sum(x^2) )
  
  # Sample from full conditional for beta1 and store it in the vector
  beta1[t] <- rnorm(1, mean=beta1.mean, sd=sqrt(beta1.var))
  
  # Shape paramter for sigmasq full conditional
  cond.shape = a0+(n/2)
  cond.rate = b0 + sum((y-beta0[t]-beta1[t]*x)^2)/2
  
  # Sample from full conditional for sigmasq and store it in the vector
  sigmasq[t] <- 1/rgamma(1, shape=cond.shape, rate=cond.rate)
  
}


###############################
###############################
## Summarize the output from ##  
## the Gibbs sampler         ##
###############################
###############################

# save beta0, beta1, sigmasq, but do not save the initial values
# First row is posterior samples for beta0
# Second row is posterior samples for beta1
# Third row is posterior samples for sigmasq
parameter.chains <- rbind(beta0[-(1)], beta1[-(1)], sigmasq[-(1)])

# Discard the first 2500 iterations as burn-in
parameter.chains <- parameter.chains[,(burnin+1):niter]

# Plot the posterior distribution for pi(beta0|y)
hist(parameter.chains[1,],breaks=40,xlab=expression(paste(beta[0])),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", beta[0],"| y)")))

# Plot the posterior distribution for pi(beta1|y)
hist(parameter.chains[2,],breaks=40,xlab=expression(paste(beta[1])),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", beta[1],"| y)")))

# Plot the posterior distribution for pi(sigmasq|y)
hist(parameter.chains[3,],breaks=40,xlab=expression(paste(sigma^2)),ylab="",yaxt="n",
     main=expression(paste("Posterior for ", pi,"(", sigma^2,"| y)")))

## Make pdf's of the plots
# pdf(file="SLRgibbs_regression_coefficients_plots.pdf", width=8, height=8)
# par(mfrow=c(2,1))
# hist(parameter.chains[1,],breaks=40,xlab=expression(paste(beta[0])),ylab="",yaxt="n",
#      main=expression(paste("Posterior for ", pi,"(", beta[0],"| y)")))
# hist(parameter.chains[2,],breaks=40,xlab=expression(paste(beta[1])),ylab="",yaxt="n",
#      main=expression(paste("Posterior for ", pi,"(", beta[1],"| y)")))
# dev.off()

# pdf(file="SLRgibbs_sigma2_plot.pdf", width=8, height=8)
# hist(parameter.chains[3,],breaks=40,xlab=expression(paste(sigma^2)),ylab="",yaxt="n",
#      main=expression(paste("Posterior for ", pi,"(", sigma^2,"| y)")))
# dev.off()


########################
########################
## Summary statistics ##
########################
########################

# The 2500 saved iterations from the Gibbs sampler
beta0.saved <- parameter.chains[1,]
beta1.saved <- parameter.chains[2,]
sigmasq.saved <- parameter.chains[3,]

# Report summary statistics for beta0
# Posterior mean
beta0.mean <- mean(beta0.saved)
# Posterior standard deviation
beta0.sd <- sd(beta0.saved)
# 95% posterior credible interval
beta0.posterior.cred <- quantile(beta0.saved,c(0.025,0.975))

# beta0.mean
# beta0.sd
# beta0.posterior.cred

# Report summary statistics for beta1
# Posterior mean
beta1.mean <- mean(beta1.saved)
# Posterior standard deviation
beta1.sd <- sd(beta1.saved)
# 95% posterior credible interval
beta1.posterior.cred <- quantile(beta1.saved,c(0.025,0.975))

beta1.posterior.cred2<- quantile(beta1.saved,c(0.005,0.995))

beta1.mean
beta1.sd
beta1.posterior.cred

# Report summary statistics for sigmasq
# Posterior mean
sigmasq.mean <- mean(sigmasq.saved)
# Posterior standard deviation
sigmasq.sd <- sd(sigmasq.saved)
# 95% posterior credible interval
sigmasq.posterior.cred <- quantile(sigmasq.saved,c(0.025,0.975))

sigmasq.mean
sigmasq.sd
sigmasq.posterior.cred


###################################
###################################
## Check convergence diagonistcs ##
###################################
###################################

# For chains 2 and 3, use different initial values:
# Initialize values for algorithm far away from the chain 1
beta0.chain2.init <- -100
beta1.chain2.init <- 100
sigmasq.chain2.init <- 0.01

beta0.chain3.init <- -100
beta1.chain3.init <- 100
sigmasq.chain3.init <- 100

# Set seed to get same values each time running this script
set.seed(123456)

# Vectors to hold the MCMC output
beta0.chain2 <- rep(NA,niter+1)
beta1.chain2 <- rep(NA,niter+1)
sigmasq.chain2 <- rep(NA,niter+1)
beta0.chain3 <- rep(NA,niter+1)
beta1.chain3 <- rep(NA,niter+1)
sigmasq.chain3 <- rep(NA,niter+1)

# Set the first value of the chains to their respective initial value
beta0.chain2[1] <- beta0.chain2.init
beta1.chain2[1] <- beta1.chain2.init
sigmasq.chain2[1] <- sigmasq.chain2.init

beta0.chain3[1] <- beta0.chain3.init
beta1.chain3[1] <- beta1.chain3.init
sigmasq.chain3[1] <- sigmasq.chain3.init

# Run the Gibbs sampler for the other chains initialized at different values
for (t in 2:(niter+1)){
  
  # mean for beta0 full conditional
  beta0.chain2.mean <- ( tau0sq*sum(y-beta1.chain2[t-1]*x) )/( sigmasq.chain2[t-1]+n*tau0sq )
  beta0.chain3.mean <- ( tau0sq*sum(y-beta1.chain3[t-1]*x) )/( sigmasq.chain3[t-1]+n*tau0sq )
  
  # variance for beta0 full conditional
  beta0.chain2.var <- ( sigmasq.chain2[t-1]*tau0sq )/( sigmasq.chain2[t-1]+n*tau0sq )
  beta0.chain3.var <- ( sigmasq.chain3[t-1]*tau0sq )/( sigmasq.chain3[t-1]+n*tau0sq )
  
  # Sample from full conditional for beta0 and store it in the vector
  beta0.chain2[t] <- rnorm(1, mean=beta0.chain2.mean, sd=sqrt(beta0.chain2.var))
  beta0.chain3[t] <- rnorm(1, mean=beta0.chain3.mean, sd=sqrt(beta0.chain3.var))
  
  # mean for beta1 full conditional
  beta1.chain2.mean <- ( tau1sq*sum(x*(y-beta0.chain2[t])) )/( sigmasq.chain2[t-1]+tau1sq*sum(x^2))
  beta1.chain3.mean <- ( tau1sq*sum(x*(y-beta0.chain3[t])) )/( sigmasq.chain3[t-1]+tau1sq*sum(x^2))

  # Variance for beta1 full conditional
  beta1.chain2.var <- ( sigmasq.chain2[t-1]*tau1sq )/( sigmasq.chain2[t-1]+tau1sq*sum(x^2) )
  beta1.chain3.var <- ( sigmasq.chain3[t-1]*tau1sq )/( sigmasq.chain3[t-1]+tau1sq*sum(x^2) )
  
  # Sample from full conditional for beta1 and store it in the vector
  beta1.chain2[t] <- rnorm(1, mean=beta1.chain2.mean, sd=sqrt(beta1.chain2.var))
  beta1.chain3[t] <- rnorm(1, mean=beta1.chain3.mean, sd=sqrt(beta1.chain3.var))
  
  # Shape paramter for sigmasq full conditional
  cond.shape = a0+(n/2)
  cond.rate.chain2 = b0 + sum((y-beta0.chain2[t]-beta1.chain2[t]*x)^2)/2
  cond.rate.chain3 = b0 + sum((y-beta0.chain3[t]-beta1.chain3[t]*x)^2)/2
  
  # Sample from full conditional for sigmasq and store it in the vector
  sigmasq.chain2[t] <- 1/rgamma(1, shape=cond.shape, rate=cond.rate.chain2)
  sigmasq.chain3[t] <- 1/rgamma(1, shape=cond.shape, rate=cond.rate.chain3)
  
}

# Plot the trace plots for beta0
plot((burnin+1):niter,beta0[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
     ylab="", cex.lab=1.5, cex.axis=1.2)
lines(beta0.chain2,col="blue")
lines(beta0.chain3,col="green")
title(main=expression(paste("Trace plot for ", beta[0])))

# Plot the trace plots for beta1
plot((burnin+1):niter,beta1[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
     ylab="", cex.lab=1.5, cex.axis=1.2)
lines(beta1.chain2,col="blue")
lines(beta1.chain3,col="green")
title(main=expression(paste("Trace plot for ", beta[1])))

# Plot the trace plots for sigma2
plot((burnin+1):niter,sigmasq[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
     ylab="", cex.lab=1.5, cex.axis=1.2)
lines(sigmasq.chain2,col="purple")
lines(sigmasq.chain3,col="green")
title(main=expression(paste("Trace plot for ", sigma^2)))


# Plot the chains on a single pdf
# pdf(file="SLR_traceplots.pdf", height=8, width=8)
# par(mfrow=c(3,1))
# 
# plot((burnin+1):niter,beta0[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
#      ylab="", cex.lab=1.5, cex.axis=1.2)
#      lines(beta0.chain2,col="blue")
#      lines(beta0.chain3,col="green")
#      title(main=expression(paste("Trace plot for ", beta[0])))
#  
# plot((burnin+1):niter,beta1[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
#     ylab="", cex.lab=1.5, cex.axis=1.2)
#     lines(beta1.chain2,col="blue")
#     lines(beta1.chain3,col="green")
#     title(main=expression(paste("Trace plot for ", beta[1])))
#
# plot((burnin+1):niter,sigmasq[(burnin+1):niter],type="l",col="red", cex.axis = 0.5, xlab="Iteration", 
#     ylab="", cex.lab=1.5, cex.axis=1.2)
#     lines(sigmasq.chain2,col="purple")
#     lines(sigmasq.chain3,col="green")
#     title(main=expression(paste("Trace plot for ", sigma^2)))
# dev.off()